<?php
/**
 * Gestion du formulaire (assistant) de création d'une carte de territoires.
 *
 * @package SPIP\CARTES\OBJET
 */
if (!defined('_ECRIRE_INC_VERSION')) {
	return;
}

/**
 * Chargement des données :.
 *
 * @return array Tableau des données à charger par le formulaire dans l'étape 1.
 */
function formulaires_creer_carte_territoires_charger() : array {
	// Paramètres de saisie de l'étape 1 à 3:
	// -- étape 1
	$valeurs = [
		'type'       => _request('type'),
		'type_carte' => _request('type_carte'),
		'pays'       => _request('pays'),
	];
	// -- étape 2
	include_spip('inc/config');
	$filtres = lire_config('cartes_territoires/filtres', []);
	$valeurs['filtre_carte'] = _request('filtre_carte');
	foreach ($filtres as $_parametres) {
		$variable = $_parametres['variable_choix'];
		$valeurs[$variable] = _request($variable);
	}
	$valeurs['cartes'] = _request('cartes');
	// -- étape 3
	$valeurs['exclusions'] = _request('exclusions');

	// Initialisation des paramètres d'affichage de l'étape 1
	// -- Liste des types de territoire.
	$types_territoire = lire_config('territoires/types');
	foreach ($types_territoire as $_type) {
		$valeurs['_types_territoire'][$_type] = spip_ucfirst(_T("territoire:type_{$_type}"));
	}
	$valeurs['_type_territoire_defaut'] = 'country';

	// -- Types de carte et défaut.
	$valeurs['_types_carte'] = [
		'territoire' => _T('carte_territoires:type_carte_territoire'),
		'carte'      => _T('carte_territoires:type_carte_carte')
	];
	$valeurs['_type_carte_defaut'] = 'territoire';

	// -- Détermination des pays disponibles pour les types subdivision, infrasubdivision et protected_area
	//    et structuration des colonnes du choix radio.
	$select = ['t1.iso_pays', 't2.nom_usage'];
	$from = ['spip_territoires AS t1', 'spip_territoires AS t2'];
	$where = ['t2.iso_territoire=t1.iso_pays'];
	foreach (['subdivision', 'infrasubdivision', 'protected_area'] as $_type) {
		$where[1] = 't1.type=' . sql_quote($_type);
		$pays = sql_allfetsel($select, $from, $where, 't1.iso_pays');
		$valeurs['_pays'][$_type] = array_column($pays, 'nom_usage', 'iso_pays');
		$valeurs['_choix_multi_col'][$_type] = 1 + intdiv(count($valeurs['_pays'][$_type]), 10);
		if ($valeurs['_choix_multi_col'][$_type] > 3) {
			$valeurs['_choix_multi_col'][$_type] = 3;
		}
	}

	// Initialisation des paramètres d'affichage utilisés en étape 2 et 3 et mis à jour dans les vérifications
	// de l'étape 1 et 2.
	// -- Etape 2 (vérification 1)
	$valeurs['_cartes_disponibles'] = _request('_cartes_disponibles');
	$valeurs['_filtres'] = _request('_filtres');
	foreach ($filtres as $_parametres) {
		$variable = $_parametres['variable_liste'];
		$valeurs[$variable] = _request($variable);
	}
	// -- Etape 3 (vérification 2)
	$valeurs['_cartes'] = _request('_cartes');
	$valeurs['_explication_filtre'] = _request('_explication_filtre');
	$valeurs['_exclusions'] = _request('_exclusions');

	// Préciser le nombre d'étapes du formulaire
	$valeurs['_etapes'] = 3;

	return $valeurs;
}

/**
 * Vérification de l'étape 1 du formulaire :.
 *
 * @return array Message d'erreur si aucun pays choisi alors que la configuration du type de teritoire l'oblige.
 *               Sinon, chargement des champs utiles à l'étape 2 :
 */
function formulaires_creer_carte_territoires_verifier_1() : array {
	// Initialisation des erreurs de vérification.
	$erreurs = [];

	// Il est inutile de vérifier à nouveau les valeurs de l'étape 1 si on est dans une étape ultérieure
	// car on ne modifie rien. On gagne du temps en passant outre.
	include_spip('inc/config');
	if (_request('_etape') === 1) {
		// On collecte donc les paramètres de l'étape 1.
		$type_carte = _request('type_carte');
		$type = _request('type');
		$pays = _request('pays');

		if ($type_carte === 'carte') {
			// On recherche si il existe des cartes du même type qui pourrait être incluse dans
			// la carte en cours de création
			include_spip('inc/carte_territoires');
			$cartes = carte_territoires_repertorier($type);
			if ($cartes) {
				$cartes = array_column($cartes, 'titre', 'id_carte');
				set_request('_cartes_disponibles', $cartes);
			} else {
				$erreurs['message_erreur'] = _T('cartes_territoires:erreur_type_territoire_0_carte');
			}
		} elseif (
			lire_config("territoires/{$type}/populated_by_country", false)
			and !$pays
		) {
			$erreurs['pays'] = _T('info_obligatoire');
		} else {
			// Le but de l'étape 2 est de filtrer l'unité de peuplement des territoires choisie en étape 1
			// par niveau, par parenté ou par catégorie suivant le type de territoires.
			// -- on initialise les listes correspondantes même si pour certains types elles resteront vides
			$listes = [];

			// -- on lit la configuration des filtres
			$filtres = lire_config('cartes_territoires/filtres', []);

			// -- on adapte la recherche à la configuration du type
			$criteres_autorises = [];
			$from = ['spip_territoires AS t1'];
			$select = [];
			$where = ['t1.type=' . sql_quote($type)];
			foreach ($filtres as $_critere => $_parametres) {
				// Initialisation de la liste de chaque critère indépendamment de son utilisation
				$listes[$_critere] = [];
				if (in_array($type, $_parametres['types_compatibles'])) {
					// Mémorisation de l'utilisation du critère : on construit aussi une liste affichable
					$criteres_autorises[$_critere] = _T('cartes_territoires:label_filtre_' . $_critere);
					// Calcul des conditions de la requête SQL
					$select[] = 't1.' . $_parametres['champ_id'];
					if ($_critere === 'parent') {
						$select[] = 't2.nom_usage';
						$from[] = 'spip_territoires AS t2';
						$where[] = 't2.iso_territoire=t1.iso_parent';
					}
				}
			}
			if ($pays) {
				$where[] = 't1.iso_pays=' . sql_quote($pays);
			}

			// -- on récupère les territoires et on construit les listes
			$territoires = sql_allfetsel($select, $from, $where, '', 't1.profondeur_type');
			foreach ($territoires as $_territoire) {
				foreach (array_keys($criteres_autorises) as $_critere) {
					$champ = $filtres[$_critere]['champ_id'];
					$champ_nom = $filtres[$_critere]['champ_nom'];
					$cle = $_territoire[$champ];
					if (!isset($listes[$_critere][$cle])) {
						$listes[$_critere][$cle] = $champ_nom
							? $_territoire[$champ_nom]
							: cartes_territoires_traduire_champ($champ, $_territoire[$champ]);
					}
				}
			}

			// -- on transmet les listes au formulaire
			set_request('_filtres', $criteres_autorises);
			foreach ($filtres as $_critere => $_parametres) {
				set_request($_parametres['variable_liste'], $listes[$_critere]);
			}
		}
	}

	return $erreurs;
}

/**
 * Vérification de l'étape 2 du formulaire :
 *
 * @return array Message d'erreur si
 */
function formulaires_creer_carte_territoires_verifier_2() : array {
	// Initialisation des erreurs de vérification.
	$erreurs = [];

	if (_request('_etape') === 2) {
		// On collecte donc les paramètres de l'étape 1.
		$type_carte = _request('type_carte');
		$type = _request('type');
		$pays = _request('pays');

		if ($type_carte === 'carte') {
			$cartes = _request('cartes');
			set_request('_cartes', $cartes);
		} else {
			// On cherche la liste des territoires constitutifs de la carte en cours de construction
			// -- initialiser les filtres provenant de l'étape 1
			$filtres = ['type' => $type];
			if ($pays) {
				$filtres['iso_pays'] = $pays;
			}
			// -- collecter les paramètres de l'étape 2, sachant qu'un seul filtre peut être choisi.
			include_spip('inc/config');
			$config_filtres = lire_config('cartes_territoires/filtres', []);
			$explication_filtre = [];
			if (
				($critere = _request('filtre_carte'))
				and ($valeur = _request($config_filtres[$critere]['variable_choix']))
			) {
				$champ = $config_filtres[$critere]['champ_id'];
				$filtres[$champ] = is_array($valeur)
					? implode(',', $valeur)
					: $valeur;
				$explication_filtre = [
					'critere' => _T('carte_territoires:champ_parametres_' . $critere . '_label'),
					'valeur'  => $filtres[$champ]
				];
			}
			// -- Lister les territoires pour les proposer à l'exclusion
			include_spip('inc/territoire');
			$territoires = territoire_repertorier($filtres);
			$territoires = array_column($territoires, 'nom_usage', 'iso_territoire');

			// Transmettre les variables au formulaire
			set_request('_explication_filtre', $explication_filtre);
			set_request('_exclusions', $territoires);
		}
	}

	return $erreurs;
}

/**
 * Exécution du formulaire : .
 *
 * @return array Tableau retourné par le formulaire contenant toujours un message de bonne exécution ou
 *               d'erreur. L'indicateur editable est toujours à vrai.
 */
function formulaires_creer_carte_territoires_traiter() : array {
	// Initialisation du retour de la fonction
	$retour = [];

	// Récupération des paramètres de base de la carte et début de consolidation des champs de l'objet carte
	$type_carte = _request('type_carte');
	$type = _request('type');
	$titre = _request('titre');
	$carte = [
		'titre'           => $titre,
		'type_carte'      => $type_carte,
		'type_territoire' => $type,
		'parametres'      => json_encode([]),
	];
	if ($type_carte === 'carte') {
		$cartes = _request('cartes');
		if ($cartes) {
			$carte['parametres'] = json_encode(['id_carte' => $cartes]);
		}
	} else {
		// Ajouter le pays à la carte si besoin
		$parametres = [];
		$pays = _request('pays');
		if ($pays) {
			$carte['iso_pays'] = $pays;
		}

		// Collecter les paramètres supplémentaires éventuellement choisi.
		include_spip('inc/config');
		$config_filtres = lire_config('cartes_territoires/filtres', []);
		if (
			($critere = _request('filtre_carte'))
			and ($valeur = _request($config_filtres[$critere]['variable_choix']))
		) {
			$champ = $config_filtres[$critere]['champ_id'];
			$parametres[$champ] = $valeur;
		}

		// Collecter les éventuelles exclusions
		$exclusions = _request('exclusions');
		if ($exclusions) {
			$parametres['exclusions'] = $exclusions;
		}

		// On encode les paramètres en JSON
		$carte['parametres'] = json_encode($parametres);
	}

	// Insertion de la carte dans la table idoine.
	include_spip('action/editer_objet');
	$id_carte = objet_inserer('carte_territoires', null, $carte);
	if (!$id_carte) {
		$retour['message_erreur'] = _T('cartes_territoires:erreur_creation_carte');
	} else {
		// Redirection vers la page d'édition du taxon
		$retour['redirect'] = parametre_url(generer_url_ecrire('carte_territoires_edit'), 'id_carte', $id_carte);
	}

	return $retour;
}

/**
 * Renvoie la traduction de la valeur d'un champ `categorie` ou `profondeur_type` de la table des territoires.
 *
 * @param string     $champ  Nom du champ de la table des territoires.
 * @param int|string $valeur Valeur du champ pour le territoire concerné.
 *
 * @return string Traduction du champ pour le territoire concerné
 */
function cartes_territoires_traduire_champ(string $champ, $valeur) : string {
	// Initialisation de la traduction
	$texte = '';

	if ($champ === 'categorie') {
		$texte = spip_ucfirst(_T('territoire:categorie_' . $valeur));
	} elseif ($champ === 'profondeur_type') {
		$texte = spip_ucfirst(_T('cartes_territoires:profondeur_' . (string) $valeur));
	}

	return $texte;
}
